@@ -265,9 +265,9 @@ DBusHandlerResult DBusMonitor::monitor_filter_func (DBusConnection     *connecti
 
 bool shouldExit1 = false;
 void DBusMonitor::run()
-{
+{    
     qDebug() << "DBus Monitor Running";
-    while(!shouldExit1 && dbus_connection_read_write_dispatch(connection, 5000));
+    while(!shouldExit1 && dbus_connection_read_write_dispatch(connection, 2));
     qDebug() << "DBus Monitor Stopping";
 }
 
@@ -291,6 +291,8 @@ bool DBusMonitor::AddFilter(QString filter)
     data = filter.toAscii().data();
     qDebug() << "    DATA:" << data;
 
+    dbus_error_init (&error);
+
     dbus_bus_add_match (connection, filter.toAscii().data(), &error);
 
     if (dbus_error_is_set (&error))
@@ -301,3 +303,54 @@ bool DBusMonitor::AddFilter(QString filter)
     }
     return true;
 }
+
+bool DBusMonitor::RemoveFilter(QString filter)
+{
+    qDebug() << "  Removing Filter:" << filter;
+
+    char* data;
+    data = filter.toAscii().data();
+    qDebug() << "    DATA:" << data;
+
+
+    dbus_error_init (&error);
+
+    dbus_bus_remove_match (connection, filter.toAscii().data(), &error);
+
+    if (dbus_error_is_set (&error))
+    {
+	qDebug() << "  Failed to setup match filter";
+	dbus_error_free (&error);
+	return false;
+    }
+    return true;
+}
+
+void DBusMonitor::Refresh(QStringList filters)
+{
+    dbus_connection_remove_filter(connection, filter_func, NULL);
+
+
+    filter_func = monitor_filter_func;
+
+    dbus_error_init (&error);
+
+    connection = dbus_bus_get (type, &error);
+
+    if (connection == NULL)
+    {
+	qDebug() << "Failed to open connection";
+	dbus_error_free (&error);
+
+    }
+
+    foreach(QString filter, filters)
+    {
+	AddFilter(filter);
+    }
+
+    if (!dbus_connection_add_filter (connection, filter_func, NULL, NULL))
+    {
+	qDebug() << "Failed to add message filter";
+    }
+}
